==> org-lsst-ccs-ui-4.2.5-sources.jar: org/lsst/ccs/gconsole/plugins/trending/timeselection/TimeWindowSelector.java <==
package org.lsst.ccs.gconsole.plugins.trending.timeselection;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.EventObject;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.CopyOnWriteArrayList;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.ListCellRenderer;
import javax.swing.SwingUtilities;
import org.freehep.application.studio.Studio;

/**
 * GUI for selecting time window for trending plots.
 * Provides a way to select time window from a list of presets and to open custom time window
 * creation and preset management dialogs, as well as access the currently selected time window.
 *
 * @author onoprien
 */
final public class TimeWindowSelector extends JComboBox {
    
// -- Private parts : ----------------------------------------------------------
       
    private final String CUSTOM_BUTTON = "Choose...";
    private final String PRESETS_BUTTON = "Presets...";
    private final String DEFAULT_NAME = "custom";
    private final String[] STANDARD_WINDOWS = {"Last Hour", "Last 6 Hours", "Last 24 Hours"};
    private final int MAX_PRESETS = 10; // Maximum number of presets to include in drop-down list
    
    private final PresetList timeWindowList;
    private volatile TimeWindow selection;
    
    private final CopyOnWriteArrayList<Listener> listeners = new CopyOnWriteArrayList<>();
    

// -- Construction and initialization : ----------------------------------------
    
    public TimeWindowSelector(Studio application) {
        
        timeWindowList = new PresetList(application);
        
        setRenderer(new Renderer(getRenderer()));
        setMaximumRowCount(MAX_PRESETS + 5);
        
        Model model = new Model(timeWindowList);
        setModel(model);
        setSelectedIndex(1);
        selection = model.getSelectedItem();
        
        addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                TimeWindow ts = myModel().getSelectedItem();
                
                switch (ts.getName()) {
                    
                    case CUSTOM_BUTTON: // custom time window has been entered
                        
                        ts = TimeWindowDialog.editTimeWindow(TimeWindowSelector.this, selection == null ? null : new TimeWindow(selection));
                        if (ts == null) {
                            setSelectedItem(selection);
                        } else {
                            if (ts.getName().isEmpty()) {
                                ts.setName(DEFAULT_NAME);
                                if (DEFAULT_NAME.equals(selection.getName())) {
                                    myModel().removeElementAt(4);
                                }
                                myModel().insertElementAt(ts, 4);
                                setSelectedIndex(4);
                            } else {
                                timeWindowList.insert(ts);
                                Model model = new Model(timeWindowList);
                                model.setSelectedItem(ts);
                                setModel(model);
                            }
                            setSelection(ts);
                        }
                        break;
                        
                    case PRESETS_BUTTON: // one of the presets has been selected through presets dialog
                        
                        ts = PresetsDialog.managePresets(TimeWindowSelector.this, timeWindowList);
                        if (ts == null) ts = selection;
                        ts.touch();
                        Model model = new Model(timeWindowList);
                        int index = model.getIndexOf(ts);
                        if (index == -1) index = 1;
                        setSelection(model.getElementAt(index));
                        model.setSelectedItem(selection);
                        setModel(model);
                        break;
                        
                    default: // one of the presets has been selected from drop-down list
                        
                        if (!DEFAULT_NAME.equals(ts.getName())) {
                            ts.touch();
                            if (DEFAULT_NAME.equals(selection.getName())) {
                                myModel().removeElementAt(4);
                            }
                        }
                        setSelection(ts);
                        
                }
            }
        });
        
    }
    
// -- Getters and setters : ----------------------------------------------------
    
    /** Returns currently selected time window. */
    public TimeWindow getSelectedTimeWindow() {
        return selection;
    }
   
    /**
     * Sets selected time window.
     * If the specified window has a name, the window should already be present in the list of presets.
     * 
     * @param timeWindow The time window to be selected.
     */
    public void setSelectedTimeWindow(TimeWindow timeWindow) {
        String name = timeWindow.getName();
        if (name == null || name.equals("")) {
            myModel().setSelectedItem(myModel().getElementAt(0));
        } else {
            int i = myModel().getIndexOf(timeWindow);
            if (i >= 0) {
                myModel().setSelectedItem(timeWindow);
            } else {
                timeWindow.touch();
                Model model = new Model(timeWindowList);
                model.setSelectedItem(timeWindow);
                setModel(model);
            }
        }
        setSelection(timeWindow);
    }
    
    /** 
     * Returns {@code PresetList} instance that maintains a list of time 
     * window presets currently known to the application.
     */
    public PresetList getPresetList() {
        return timeWindowList;
    }
    
    /**
     * Returns time window with the specified name.
     * If the given name is not known, treats it as a compressed string defining a time window,
     * and constructs a new {@code TimeWindow} instance. If the created instance has a non-empty
     * name, adds it to the list of presets.
     * 
     * @param name Name of the window, or a string defining a time window.
     * @return The requested time window, or {@code null} if it does not exist and cannot be created.
     */
    public TimeWindow getTimeWindow(String name) {
        if (name == null) return getSelectedTimeWindow();
        for (int i = 0; i<STANDARD_WINDOWS.length; i++) {
            if (STANDARD_WINDOWS[i].equals(name)) {
                return myModel().getElementAt(i+1);
            }
        }
        TimeWindow timeWindow = timeWindowList.get(name);
        if (timeWindow == null) {
            try {
                timeWindow = TimeWindow.parseCompressedString(name);
                name = timeWindow.getName();
                if (name != null && !name.equals("")) {
                    timeWindowList.insert(timeWindow);
                }
            } catch (IllegalArgumentException x) {
            }
        }
        return timeWindow;
    }
    
    
// -- Handling listeners : -----------------------------------------------------
    
    public interface Listener {
        void timeWindowChanged(Event event);
    }
    
    static public class Event extends EventObject {

        private final TimeWindow timeWindow;

        public Event(Object source, TimeWindow timeWindow) {
            super(source);
            this.timeWindow =  timeWindow;
        }

        public TimeWindow getTimeWindow() {
            return timeWindow;
        }
        
    }
    
    public void addListener(Listener listener) {
        listeners.add(listener);
        if (selection != null) {
            Event event = new Event(this, selection);
            SwingUtilities.invokeLater(() -> {
                listener.timeWindowChanged(event);
            });
        }
    }
    
    public void removeListener(Listener listener) {
        listeners.remove(listener);
    }
    
    public void removeAllListeners() {
        listeners.clear();
    }
    
    private void setSelection(TimeWindow timeWindow) {
        if (!Objects.equals(selection, timeWindow)) {
            selection = timeWindow;
            Event event = new Event(this, selection);
            listeners.forEach(listener -> listener.timeWindowChanged(event));
        }
    }
    
    
// -- Local methods : ----------------------------------------------------------
    
    private Model myModel() {
        return (Model) super.getModel();
    }


// -- Custom renderer : --------------------------------------------------------
    
    private class Renderer implements ListCellRenderer {

        private final ListCellRenderer horse;
        private final JPanel separatorPanel = new JPanel(new BorderLayout());
        private final JSeparator separator = new JSeparator();

        Renderer(ListCellRenderer renderer) {
            horse = renderer;
        }

        @Override
        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
            Component comp = horse.getListCellRendererComponent(list, ((TimeWindow)value).getName(), index, isSelected, cellHasFocus);
            if (index == 0 || index == 3) {
                separatorPanel.removeAll();
                separatorPanel.add(comp, BorderLayout.CENTER);
                separatorPanel.add(separator, BorderLayout.SOUTH);
                comp = separatorPanel;
            } else if (index > 4 && index == TimeWindowSelector.this.getItemCount()-1) {
                separatorPanel.removeAll();
                separatorPanel.add(comp, BorderLayout.CENTER);
                separatorPanel.add(separator, BorderLayout.NORTH);
                comp = separatorPanel;
            }
            return comp;
        }

    }
    
// -- Custom model : -----------------------------------------------------------
    
    private class Model extends DefaultComboBoxModel<TimeWindow> {
        
        Model(PresetList tsList) {

        // Add element for launching time range construction dialog :
            
            addElement(new TimeWindow(CUSTOM_BUTTON));

        // Add standard ranges :
            
            TimeWindow selectedItem = new TimeWindow(STANDARD_WINDOWS[0], "now-3600", "now", false);
            addElement(selectedItem);
            addElement(new TimeWindow(STANDARD_WINDOWS[1], "now-21600", "now", false));
            addElement(new TimeWindow(STANDARD_WINDOWS[2], "now-86400", "now", false));

        // Add custom ranges :
            
            List<TimeWindow> customRanges = tsList.getRecent(MAX_PRESETS);

            for (TimeWindow ts : customRanges) {
                addElement(ts);
            }

        // Add element for launching time range presets dialog :
            
            addElement(new TimeWindow(PRESETS_BUTTON));

        // Set default selection :
            
            setSelectedItem(selectedItem);
        }
    
        @Override
        public TimeWindow getSelectedItem() {
            return (TimeWindow) super.getSelectedItem();
        }
        
    }
    
}

==> org-lsst-ccs-ui-4.3.2-sources.jar: org/lsst/ccs/gconsole/plugins/trending/timeselection/TimeWindowSelector.java <==
package org.lsst.ccs.gconsole.plugins.trending.timeselection;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.EventObject;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.CopyOnWriteArrayList;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.ListCellRenderer;
import javax.swing.SwingUtilities;
import org.freehep.application.studio.Studio;

/**
 * GUI for selecting time window for trending plots.
 * Provides a way to select time window from a list of presets and to open custom time window
 * creation and preset management dialogs, as well as access the currently selected time window.
 *
 * @author onoprien
 */
final public class TimeWindowSelector extends JComboBox {
    
// -- Private parts : ----------------------------------------------------------
       
    private final String CUSTOM_BUTTON = "Choose...";
    private final String PRESETS_BUTTON = "Presets...";
    private final String DEFAULT_NAME = "custom";
    private final String[] STANDARD_WINDOWS = {"Last Hour", "Last 6 Hours", "Last 24 Hours"};
    private final int MAX_PRESETS = 10; // Maximum number of presets to include in drop-down list
    
    private final PresetList timeWindowList;
    private volatile TimeWindow selection;
    
    private final CopyOnWriteArrayList<Listener> listeners = new CopyOnWriteArrayList<>();
    

// -- Construction and initialization : ----------------------------------------
    
    public TimeWindowSelector(Studio application) {
        
        timeWindowList = new PresetList(application);
        
        setRenderer(new Renderer(getRenderer()));
        setMaximumRowCount(MAX_PRESETS + 5);
        
        Model model = new Model(timeWindowList);
        setModel(model);
        setSelectedIndex(1);
        selection = model.getSelectedItem();
        
        addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                TimeWindow ts = myModel().getSelectedItem();
                
                switch (ts.getName()) {
                    
                    case CUSTOM_BUTTON: // custom time window has been entered
                        
                        ts = TimeWindowDialog.editTimeWindow(TimeWindowSelector.this, selection == null ? null : new TimeWindow(selection));
                        if (ts == null) {
                            setSelectedItem(selection);
                        } else {
                            if (ts.getName().isEmpty()) {
                                ts.setName(DEFAULT_NAME);
                                if (DEFAULT_NAME.equals(selection.getName())) {
                                    myModel().removeElementAt(4);
                                }
                                myModel().insertElementAt(ts, 4);
                                setSelectedIndex(4);
                            } else {
                                timeWindowList.insert(ts);
                                Model model = new Model(timeWindowList);
                                model.setSelectedItem(ts);
                                setModel(model);
                            }
                            setSelection(ts);
                        }
                        break;
                        
                    case PRESETS_BUTTON: // one of the presets has been selected through presets dialog
                        
                        ts = PresetsDialog.managePresets(TimeWindowSelector.this, timeWindowList);
                        if (ts == null) ts = selection;
                        ts.touch();
                        Model model = new Model(timeWindowList);
                        int index = model.getIndexOf(ts);
                        if (index == -1) index = 1;
                        setSelection(model.getElementAt(index));
                        model.setSelectedItem(selection);
                        setModel(model);
                        break;
                        
                    default: // one of the presets has been selected from drop-down list
                        
                        if (!DEFAULT_NAME.equals(ts.getName())) {
                            ts.touch();
                            if (DEFAULT_NAME.equals(selection.getName())) {
                                myModel().removeElementAt(4);
                            }
                        }
                        setSelection(ts);
                        
                }
            }
        });
        
    }
    
// -- Getters and setters : ----------------------------------------------------
    
    /** Returns currently selected time window. */
    public TimeWindow getSelectedTimeWindow() {
        return selection;
    }
   
    /**
     * Sets selected time window.
     * If the specified window has a name, the window should already be present in the list of presets.
     * 
     * @param timeWindow The time window to be selected.
     */
    public void setSelectedTimeWindow(TimeWindow timeWindow) {
        String name = timeWindow.getName();
        if (name == null || name.equals("")) {
            myModel().setSelectedItem(myModel().getElementAt(0));
        } else {
            int i = myModel().getIndexOf(timeWindow);
            if (i >= 0) {
                myModel().setSelectedItem(timeWindow);
            } else {
                timeWindow.touch();
                Model model = new Model(timeWindowList);
                model.setSelectedItem(timeWindow);
                setModel(model);
            }
        }
        setSelection(timeWindow);
    }
    
    /** 
     * Returns {@code PresetList} instance that maintains a list of time 
     * window presets currently known to the application.
     */
    public PresetList getPresetList() {
        return timeWindowList;
    }
    
    /**
     * Returns time window with the specified name.
     * If the given name is not known, treats it as a compressed string defining a time window,
     * and constructs a new {@code TimeWindow} instance. If the created instance has a non-empty
     * name, adds it to the list of presets.
     * 
     * @param name Name of the window, or a string defining a time window.
     * @return The requested time window, or {@code null} if it does not exist and cannot be created.
     */
    public TimeWindow getTimeWindow(String name) {
        if (name == null) return getSelectedTimeWindow();
        for (int i = 0; i<STANDARD_WINDOWS.length; i++) {
            if (STANDARD_WINDOWS[i].equals(name)) {
                return myModel().getElementAt(i+1);
            }
        }
        TimeWindow timeWindow = timeWindowList.get(name);
        if (timeWindow == null) {
            try {
                timeWindow = TimeWindow.parseCompressedString(name);
                name = timeWindow.getName();
                if (name != null && !name.equals("")) {
                    timeWindowList.insert(timeWindow);
                }
            } catch (IllegalArgumentException x) {
            }
        }
        return timeWindow;
    }
    
    
// -- Handling listeners : -----------------------------------------------------
    
    public interface Listener {
        void timeWindowChanged(Event event);
    }
    
    static public class Event extends EventObject {

        private final TimeWindow timeWindow;

        public Event(Object source, TimeWindow timeWindow) {
            super(source);
            this.timeWindow =  timeWindow;
        }

        public TimeWindow getTimeWindow() {
            return timeWindow;
        }
        
    }
    
    public void addListener(Listener listener) {
        listeners.add(listener);
//        if (selection != null) { // FIXME: commented out to remove double queries at auto-startup. Needs to be verified.
//            Event event = new Event(this, selection);
//            SwingUtilities.invokeLater(() -> {
//                listener.timeWindowChanged(event);
//            });
//        }
    }
    
    public void removeListener(Listener listener) {
        listeners.remove(listener);
    }
    
    public void removeAllListeners() {
        listeners.clear();
    }
    
    private void setSelection(TimeWindow timeWindow) {
        if (!Objects.equals(selection, timeWindow)) {
            selection = timeWindow;
            Event event = new Event(this, selection);
            listeners.forEach(listener -> listener.timeWindowChanged(event));
        }
    }
    
    
// -- Local methods : ----------------------------------------------------------
    
    private Model myModel() {
        return (Model) super.getModel();
    }


// -- Custom renderer : --------------------------------------------------------
    
    private class Renderer implements ListCellRenderer {

        private final ListCellRenderer horse;
        private final JPanel separatorPanel = new JPanel(new BorderLayout());
        private final JSeparator separator = new JSeparator();

        Renderer(ListCellRenderer renderer) {
            horse = renderer;
        }

        @Override
        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
            Component comp = horse.getListCellRendererComponent(list, ((TimeWindow)value).getName(), index, isSelected, cellHasFocus);
            if (index == 0 || index == 3) {
                separatorPanel.removeAll();
                separatorPanel.add(comp, BorderLayout.CENTER);
                separatorPanel.add(separator, BorderLayout.SOUTH);
                comp = separatorPanel;
            } else if (index > 4 && index == TimeWindowSelector.this.getItemCount()-1) {
                separatorPanel.removeAll();
                separatorPanel.add(comp, BorderLayout.CENTER);
                separatorPanel.add(separator, BorderLayout.NORTH);
                comp = separatorPanel;
            }
            return comp;
        }

    }
    
// -- Custom model : -----------------------------------------------------------
    
    private class Model extends DefaultComboBoxModel<TimeWindow> {
        
        Model(PresetList tsList) {

        // Add element for launching time range construction dialog :
            
            addElement(new TimeWindow(CUSTOM_BUTTON));

        // Add standard ranges :
            
            TimeWindow selectedItem = new TimeWindow(STANDARD_WINDOWS[0], "now-3600", "now", false);
            addElement(selectedItem);
            addElement(new TimeWindow(STANDARD_WINDOWS[1], "now-21600", "now", false));
            addElement(new TimeWindow(STANDARD_WINDOWS[2], "now-86400", "now", false));

        // Add custom ranges :
            
            List<TimeWindow> customRanges = tsList.getRecent(MAX_PRESETS);

            for (TimeWindow ts : customRanges) {
                addElement(ts);
            }

        // Add element for launching time range presets dialog :
            
            addElement(new TimeWindow(PRESETS_BUTTON));

        // Set default selection :
            
            setSelectedItem(selectedItem);
        }
    
        @Override
        public TimeWindow getSelectedItem() {
            return (TimeWindow) super.getSelectedItem();
        }
        
    }
    
}
